<?php
require_once __DIR__.'/util.php';
require_once __DIR__.'/../../api/nachos.php';
require_once __DIR__.'/../../api/lib/request.php';
require_once __DIR__.'/../../api/lib/response.php';
require_once __DIR__.'/../../api/lib/error_response.php';

class NachosTest extends PHPUnit_Framework_TestCase {
	public function test_full_export_valid() {
		$request = new MockRequest(array('export_type'=>'full_export'), 'POST');
		$data = '{"creation_time":"2018-05-29 07:32:42.544838","export_type":"full_export","id":' .
				'"7705eb78-d0b6-4b15-8016-a165c2db8052","status":"pending","user":"monitor"}';
		# Mock the method that does the actual CURL request to nachos
		$MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('curl_request'))->getMock();
		$MockNachos->expects(
			$this->once())->method('curl_request')->will(
			$this->returnValue(array(
				'response' => $data,
				'http_code' => 202,
				'content_type' => 'application/json'
			)));

		$response = $MockNachos->handle_request($request);

		$this->assertInstanceOf('Api_Response', $response);
		$this->assertSame(202, $response->get_code());
		$this->assertSame($data, $response->get_body());
	}

	public function test_user_export_valid() {
		$request = new MockRequest(array('export_type'=>'user_export'), 'POST');
		$data = '{"creation_time":"2018-05-29 07:32:42.544838","export_type":"full_export","id":' .
				'"7705eb78-d0b6-4b15-8016-a165c2db8052","status":"pending","user":"monitor"}';
		$MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('curl_request'))->getMock();
		$MockNachos->expects(
			$this->once())->method('curl_request')->will(
			$this->returnValue(array(
				'response' => $data,
				'http_code' => 202,
				'content_type' => 'application/json'
			)));

		$response = $MockNachos->handle_request($request);

		$this->assertInstanceOf('Api_Response', $response);
		$this->assertSame(202, $response->get_code());
		$this->assertSame($data, $response->get_body());
	}

	public function test_nachos_unresponsive() {
		$request = new MockRequest(array('export_type'=>'full_export'), 'POST');
		$MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('curl_request'))->getMock();
		$MockNachos->expects(
			$this->once())->method('curl_request')->will(
			$this->returnValue(array(
				'response' => '',
				'http_code' => 0,
				'content_type' => 'application/json'
			)));

		$this->setExpectedException('Api_Error_Response', '', 502);
		$MockNachos->handle_request($request);
	}


	public function test_get_latest_export() {
		$request = new MockRequest(array(), 'GET');
		$data = '{"creation_time":"2018-05-31T07:25:23.000000","export_type":"full_export","id":' .
				'"1ed3cfa0-7b9c-4d8c-9617-4136b819c87f","status":"success","status_details":[{"details":"backup ' .
				'complete","progress":1,"state":"backup"},{"details":"","progress":1,"state":"config_generation"},'.
				'{"details":"skip","progress":1,"state":"verification"},{"details":"","progress":1,"state":' .
				'"commit"}],"user":"monitor"}';
		$MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('curl_request'))->getMock();
		$MockNachos->expects(
			$this->once())->method('curl_request')->will(
			$this->returnValue(array(
				'response' => $data,
				'http_code' => 200,
				'content_type' => 'application/json'
			)));

		$response = $MockNachos->handle_request($request);

		$this->assertInstanceOf('Api_Response', $response);
		$this->assertSame(200, $response->get_code());
		$this->assertSame($data, $response->get_body());
	}

	public function test_get_latest_export_when_no_exports_exist() {
		$request = new MockRequest(array(), 'GET');
		$data = '{"description":"No export jobs could be found in the database","name":"no_exports_found_error"}';
		$MockNachos = $this->getMockBuilder('\Nachos')->setMethods(array('curl_request'))->getMock();
		$MockNachos->expects(
			$this->once())->method('curl_request')->will(
			$this->returnValue(array(
				'response' => $data,
				'http_code' => 404,
				'content_type' => 'application/json'
			)));

		$response = $MockNachos->handle_request($request);

		$this->assertInstanceOf('Api_Response', $response);
		$this->assertSame(404, $response->get_code());
		$this->assertSame($data, $response->get_body());
	}
}
